﻿using System;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using SharpSvn;

namespace SharpSvnTests
{
    public partial class Form1 : Form
    {
        private CancellationTokenSource cancellationTokenSource;
        private CancellationToken cancellationToken;

        public Form1()
        {
            this.InitializeComponent();
        }

        private async void btnCheckOut_Click(object sender, EventArgs e)
        {
            this.btnCheckOut.Enabled = false;
            try
            {
                this.cancellationTokenSource = new CancellationTokenSource();
                this.cancellationToken = this.cancellationTokenSource.Token;
                await Task.Run(() =>
                {
                    using (SvnClient client = new SvnClient())
                    {
                        client.Cancel += this.Client_Cancel;
                        client.CheckOut(new SvnUriTarget("<svnrepourl>"), "repo");

                    }
                }, this.cancellationTokenSource.Token);
            }
            catch (SvnOperationCanceledException ex)
            {
                MessageBox.Show(ex.Message);

                // NOTE: This will fail if any file handles are open in .svn\tmp
                using (SvnClient svnclient = new SvnClient())
                {
                    svnclient.CleanUp("repo", new SvnCleanUpArgs() { BreakLocks = true, ClearDavCache = true, FixTimestamps = true, VacuumPristines = true });
                }
            }
            finally
            {
                this.btnCheckOut.Enabled = true;
            }
        }

        private void Client_Cancel(object sender, SvnCancelEventArgs e)
        {
            if (this.cancellationToken.IsCancellationRequested)
            {
                e.Cancel = true;
            }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.cancellationTokenSource.Cancel();
        }
    }
}
